// nextApiRailsWrapper.js

const mimeTypes = {
  html: "text/html",
  plain: "text/plain",
  json: "application/json",
  xml: "application/xml",
  pdf: "application/pdf",
  jpeg: "image/jpeg",
  png: "image/png",
  gif: "image/gif",
  svg: "image/svg+xml",
  css: "text/css",
  javascript: "application/javascript",
  csv: "text/csv",
  excel: "application/vnd.ms-excel",
  word: "application/msword",
  powerpoint: "application/vnd.ms-powerpoint",
  zip: "application/zip",
  mp3: "audio/mpeg",
  wav: "audio/wav",
  mpeg: "video/mpeg",
  mp4: "video/mp4",
};

// Define the wrapper function
const createApiRailsHandlers = (controller, authenticationProxy) => {
  return async (req, res) => {
    const { method } = req;
    let authenticatedUser = null;
    // If the controller doesn't have the method, return 405
    const action = controller[parseUrlToMethod(req.path, method)];

    const acceptsFormat = controller.acceptsFormat || mimeTypes["json"];

    if (!action) {
      res.setHeader("Allow", Object.keys(controller));
      res.status(405).end(`Method ${method} Not Allowed`);
      return;
    }

    const requiresAuthentication =
      (controller.authenticate && controller.authenticate === true) ||
      (controller.authenticate &&
        Array.isArray(controller.authenticate) &&
        controller.authenticate.includes(action));

    if (
      requiresAuthentication &&
      authenticationProxy &&
      typeof authenticationProxy === "function"
    ) {
      authenticatedUser = authenticationProxy(req);

      if (!authenticatedUser) {
        res.status(401).end("Unauthorized");
        return;
      }
    }

    const beforeActions = controller.beforeActions || [];
    const afterActions = controller.afterActions || [];

    try {
      await runActions(beforeActions, req, res, authenticatedUser);
      await runAction(action, req, res, authenticatedUser);
      await runActions(afterActions, req, res, authenticatedUser);
    } catch (error) {
      res.status(500).end("Internal Server Error");
    }
  };
};

// Define the respondTo function
const respondTo = (mimeType, handler) => {
  return (req, res) => {
    const acceptHeader = req.headers["accept"] || mimeTypes["json"];
    if (acceptHeader && acceptHeader.includes(mimeType)) {
      res.setHeader("Content-Type", mimeType);
      handler(req, res);
    } else {
      res.status(406).end("Not Acceptable");
    }
  };
};

const runAction = async (action, req, res, authenticatedUser) => {
  if (typeof action === "function") {
    await action(req, res, authenticatedUser);
  } else if (
    typeof action === "object" &&
    typeof action["handler"] === "function"
  ) {
    await action["handler"](req, res, authenticatedUser);
  }
};

const runActions = async (actions, req, res, authenticatedUser) => {
  actions.map(async (action) => {
    await runAction(action, req, res, authenticatedUser);
  });
};

const parseUrlToMethod = (url, httpMethod) => {
  const urlParts = url.split("/");
  const controller = urlParts[0];
  switch (httpMethod) {
    case "GET":
      const lastPart = urlParts[urlParts.length - 1];
      if (lastPart === "new") {
        return `new`;
      } else if (lastPart === "edit") {
        return `edit`;
      } else if (urlParts.length === 1) {
        return `index`;
      } else {
        return `show`;
      }
    case "POST":
      return `create`;
    case "PUT":
    case "PATCH":
      return `update`;
    case "DELETE":
      return `destroy`;
    default:
      return httpMethod.toLowerCase();
  }
};

// Define the Rails-inspired interface DSL
const railsApi = (controller, authenticationProxy) => {
  return createApiRailsHandlers(controller, authenticationProxy);
};

module.exports = { railsApi, respondTo };
